@@ -24,12 +24,11 @@ from enum import IntEnum
 from construct import (
     Const,
     Default,
+    Embedded,
     GreedyRange,
     Int8ul,
     Int16ul,
-    Optional,
     Struct,
-    Switch,
     this,
 )
 
@@ -42,7 +41,9 @@ from bluetooth_mesh.messages.config import (
     UnicastUnassignedGroupAddress,
 )
 from bluetooth_mesh.messages.generic.onoff import TransitionTime
-from bluetooth_mesh.messages.util import EnumAdapter, Opcode
+from bluetooth_mesh.messages.util import EnumAdapter
+from bluetooth_mesh.messages.util import EnumSwitch as Switch
+from bluetooth_mesh.messages.util import NamedSelect, Opcode, SwitchStruct
 
 MAX_RECORD_COUNT = 32
 
@@ -79,13 +80,22 @@ RegistryRecord = Struct(
     "max_hops" / ConfigHeartbeatHops
 )
 
-NetworkDiagnosticSetupServerPublicationSet = Struct(
+NetworkDiagnosticSetupServerPublicationSetMinimal = Struct(
     "destination" / UnicastUnassignedGroupAddress,
     "count" / Int16ul,
     "period" / TransitionTime,
     "ttl" / TTL,
     "net_key_index" / Int12ul,
-    "features" / Optional(ConfigHeartbeatPublicationFeatures)
+)
+
+NetworkDiagnosticSetupServerPublicationSetOptional = Struct(
+    Embedded(NetworkDiagnosticSetupServerPublicationSetMinimal),
+    "features" / ConfigHeartbeatPublicationFeatures
+)
+
+NetworkDiagnosticSetupServerPublicationSet = NamedSelect(
+    optional=NetworkDiagnosticSetupServerPublicationSetOptional,
+    minimal=NetworkDiagnosticSetupServerPublicationSetMinimal,
 )
 
 NetworkDiagnosticSetupServerPublicationStatus = NetworkDiagnosticSetupServerPublicationSet
@@ -102,36 +112,46 @@ NetworkDiagnosticServerSubscriptionStatus = Struct(
     "record" / GreedyRange(RegistryRecord)
 )
 
-NetworkDiagnosticServerPayload = Struct(
+NetworkDiagnosticServerParams = Struct(
     "subopcode" / EnumAdapter(Int8ul, NetworkDiagnosticServerSubOpcode),
-    "payload" / Default(Switch(
+    "payload" / Switch(
         this.subopcode,
         {
             NetworkDiagnosticServerSubOpcode.SUBSCRIPTION_SET: NetworkDiagnosticServerSubscriptionSet,
             NetworkDiagnosticServerSubOpcode.SUBSCRIPTION_SET_UNACK: NetworkDiagnosticServerSubscriptionSet,
             NetworkDiagnosticServerSubOpcode.SUBSCRIPTION_STATUS: NetworkDiagnosticServerSubscriptionStatus
         }
-    ), None)
+    )
 )
 
-NetworkDiagnosticSetupServerPayload = Struct(
+NetworkDiagnosticSetupServerParams = SwitchStruct(
     "subopcode" / EnumAdapter(Int8ul, NetworkDiagnosticSetupServerSubOpcode),
-    "payload" / Default(Switch(
+    "payload" / Switch(
         this.subopcode,
         {
             NetworkDiagnosticSetupServerSubOpcode.PUBLICATION_SET: NetworkDiagnosticSetupServerPublicationSet,
             NetworkDiagnosticSetupServerSubOpcode.PUBLICATION_STATUS: NetworkDiagnosticSetupServerPublicationStatus,
         }
-    ), None)
+    )
 )
 
-NetworkDiagnosticServerMessage = Struct(
-    "opcode" / Const(NetworkDiagnosticServerOpcode.SILVAIR_NDS, Opcode(NetworkDiagnosticServerOpcode)),
-    "params" / NetworkDiagnosticServerPayload
+NetworkDiagnosticServerMessage = SwitchStruct(
+    "opcode" / Opcode(NetworkDiagnosticServerOpcode),
+    "params" / Switch(
+        this.opcode,
+        {
+            NetworkDiagnosticServerOpcode.SILVAIR_NDS: NetworkDiagnosticServerParams
+        }
+    )
 )
 
-NetworkDiagnosticSetupServerMessage = Struct(
-    "opcode" / Const(NetworkDiagnosticSetupServerOpcode.SILVAIR_NDS_SETUP, Opcode(NetworkDiagnosticSetupServerOpcode)),
-    "params" / NetworkDiagnosticSetupServerPayload
+NetworkDiagnosticSetupServerMessage = SwitchStruct(
+    "opcode" / Opcode(NetworkDiagnosticSetupServerOpcode),
+    "params" / Switch(
+        this.opcode,
+        {
+            NetworkDiagnosticSetupServerOpcode.SILVAIR_NDS_SETUP: NetworkDiagnosticSetupServerParams
+        }
+    )
 )
 # fmt: on
